-- MAT 405D: W
-- Demonstration for system of ODEs
-- Dr. Stephen Moore | stephen.moore@ucc.edu.gh
-- Department of Mathematics, University of Cape Coast

-- define the step size for the numerical algorithm
local h = 0.005
-- define given parameters
local sigma = 10
local beta = 8/3
local rho = 28
-- time and initial values [x0,y0,z0]
local tspan = {0, 20}
local y0 = {1.3, 1.2, 1.6}

-- The lorenzo function
local function lorenzo(t, y, sigma, rho, beta)
  -- This is the Lorenz differential equations
  -- dx/dt = sigma(y-x)
  -- dy/dt = rho*x-y-x*z
  -- dz/dt = -beta*z+x*y
  -- sigma, rho, beta are real values
  return {
    sigma * y[2] - sigma * y[1],
    rho * y[1] - y[2] - y[1] * y[3],
    -beta * y[3] + y[1] * y[2]
  }
end

-- call for the function lorenzo with initial values and parameters
-- there are several solvers for ode, e.g., ode23, ode45, etc.
-- help ode23 will give you more information
local t, y = ode23(lorenzo, tspan, y0, h, sigma, rho, beta)

-- Subplot allows you to plot several figures into one figure.
-- plot x against y
subplot(2, 2, 1)
plot(y[{{}, 1}], y[{{}, 2}], 'b--')
title('Fluid motion against Horizontal Temperature')

-- plot x against z
subplot(2, 2, 2)
plot(y[{{}, 1}], y[{{}, 3}], 'r--')
title('Fluid motion against Vertical Temperature')

-- plot y against z
subplot(2, 2, 3)
plot(y[{{}, 2}], y[{{}, 3}], 'g--')
title('Horizontal Temperature against Vertical Temperature')

-- phase plane plot
subplot(2, 2, 4)
plot3(y[{{}, 1}], y[{{}, 2}], y[{{}, 3}], 'r')
title('3D Phase-plane of Lorenz Equation')
xlabel('x')
ylabel('y')
zlabel('z')
grid()